package com.example.tdl.web;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.example.tdl.domain.dto.CommFeedback;
import com.example.tdl.domain.vo.*;
import com.example.tdl.service.AlarmService;

import com.example.tdl.service.TDLLogService;
import com.example.tdl.service.WarehouseService;
import com.example.tdl.service.redis.TokenRedisService;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.List;


@RestController
@RequestMapping("/alarm")
public class AlarmController {

    private Logger logger = LoggerFactory.getLogger(AlarmController.class);

    CommFeedback fb=new CommFeedback();

    Gson gson=new Gson();

    @Autowired
    private AlarmService alarmService=new AlarmService();

    @Autowired
    private WarehouseService warehouseService;

    @Autowired
    private TokenRedisService tokenRedisService;

    @Autowired
    private TDLLogService  tdlLogService;

    @Autowired
    private I18nController  i18n;

    //查询所有预警信息
    @ApiOperation(value = "查询所有预警信息",notes = "查询所有预警信息,返回值说明：" +
            "   alarmType：预警类型，" +
            "   alarmScene：预警场景，" +
            "   temMax：温度上限，" +
            "   temMin：温度下限，" +
            "   humidityMax：湿度上限，" +
            "   humidityMin：湿度下限，" +
            "   tilt：倾斜度，" +
            "   shock：震度，" +
            "   remark：备注")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/getAll",method = RequestMethod.GET)
    public Object getAll(HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        return alarmService.getAll(user.getCompanyName());
    }

    //根据预警类型模糊查询
    @ApiOperation(value = "根据预警类型模糊查询",notes = "查询所有预警信息,返回值说明：" +
            "   alarmType：预警类型，" +
            "   alarmScene：预警场景，" +
            "   temMax：温度上限，" +
            "   temMin：温度下限，" +
            "   humidityMax：湿度上限，" +
            "   humidityMin：湿度下限，" +
            "   tilt：倾斜度，" +
            "   shock：震度，" +
            "   remark：备注")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/getAllByAlarmType",method = RequestMethod.POST)
    public Object getAllByAlarmType(@RequestBody String alarmType,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        JSONObject jsonObject= JSON.parseObject(alarmType);
        alarmType=(String)jsonObject.get("alarmType");
        return alarmService.getAllByAlarmType(user.getCompanyName(),alarmType);
    }

    //添加预警信息
    @ApiOperation(value = "添加预警信息",notes = "添加预警信息,companyName不用传，需要传递参数：" +
            "   alarmType：预警类型，" +
            "   alarmScene：预警场景，" +
            "   temMax：温度上限，" +
            "   temMin：温度下限，" +
            "   humidityMax：湿度上限，" +
            "   humidityMin：湿度下限，" +
            "   tilt：倾斜度，" +
            "   shock：震度，" +
            "   remark：备注")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/addAlarm",method = RequestMethod.POST)
    public Object addAlarm(@RequestBody AddAlarmVo addAlarmVo,HttpServletRequest request){
        String port =request.getHeader("AccountLanguage");
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        if (StringUtils.isEmpty(addAlarmVo.getAlarmType())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"alarmType"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addAlarmVo.getAlarmScene())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"alarmScene"));
            return gson.toJson(fb);
        }
        if (!addAlarmVo.getAlarmScene().equals("仓库")&&!addAlarmVo.getAlarmScene().equals("运输")){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"wrongAlarmScene"));
            return gson.toJson(fb);
        }
        if (addAlarmVo.getTemMax()== null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"temMax"));
            return gson.toJson(fb);
        }
        if (addAlarmVo.getTemMin()== null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"temMin"));
            return gson.toJson(fb);
        }
        if(addAlarmVo.getTemMin() >addAlarmVo.getTemMax()){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"wrongTemp"));
            return gson.toJson(fb);
        }
        if (addAlarmVo.getHumidityMax()==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"humidityMax"));
            return gson.toJson(fb);
        }
        if (addAlarmVo.getHumidityMin()==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"humidityMin"));
            return gson.toJson(fb);
        }
        if(addAlarmVo.getHumidityMin() >addAlarmVo.getHumidityMax()){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"wrongHumidity"));
            return gson.toJson(fb);
        }
        if (addAlarmVo.getTilt()==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"tilt"));
            return gson.toJson(fb);
        }
        if (addAlarmVo.getShock()==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"shock"));
            return gson.toJson(fb);
        }
        if (alarmService.getByTypeAndScene(addAlarmVo.getAlarmType(),addAlarmVo.getAlarmScene(),user.getCompanyName())!=null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"alarmExist"));
            return gson.toJson(fb);
        }
        addAlarmVo.setCompanyName(user.getCompanyName());
        int a=alarmService.addAlarm(addAlarmVo);
        if (a>0){
            fb.setCode(1);
            fb.setMessage(i18n.getMessage(request,"addSuccess"));
        }else{
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"addFailure"));
        }
        return gson.toJson(fb);
    }

    //修改预警信息
    @ApiOperation(value = "修改预警信息",notes = "修改预警信息,companyName不用传，需要传递参数：" +
            "   oldAlarmType：需要修改的预警类型，" +
            "   oldAlarmScene：需要修改的预警场景，" +
            "   alarmType：预警类型，" +
            "   alarmScene：预警场景，" +
            "   temMax：温度上限，" +
            "   temMin：温度下限，" +
            "   humidityMax：湿度上限，" +
            "   humidityMin：湿度下限，" +
            "   tilt：倾斜度，" +
            "   shock：震度，" +
            "   remark：备注")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/updateAlarm",method = RequestMethod.POST)
    public Object updateAlarm(@RequestBody UpdateAlarmVo updateAlarmVo,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        String port = request.getHeader("AccountLanguage");
        if (StringUtils.isEmpty(updateAlarmVo.getOldAlarmType())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"modifiedAlarmType"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateAlarmVo.getOldAlarmScene())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"modifiedAlarmScene"));
            return gson.toJson(fb);
        }
        if (!updateAlarmVo.getOldAlarmScene().equals("仓库")&&!updateAlarmVo.getOldAlarmScene().equals("运输")){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"modifiedWrongAlarmScene"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateAlarmVo.getAlarmType())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"alarmType"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(updateAlarmVo.getAlarmScene())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"alarmScene"));
            return gson.toJson(fb);
        }
        if (!updateAlarmVo.getAlarmScene().equals("仓库")&&!updateAlarmVo.getAlarmScene().equals("运输")){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"wrongAlarmScene"));
            return gson.toJson(fb);
        }
        if (updateAlarmVo.getTemMax()==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"temMax"));
            return gson.toJson(fb);
        }
        if (updateAlarmVo.getTemMin()==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"temMin"));
            return gson.toJson(fb);
        }
        if(updateAlarmVo.getTemMin() >updateAlarmVo.getTemMax()){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"wrongTemp"));
            return gson.toJson(fb);
        }
        if (updateAlarmVo.getHumidityMax()==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"humidityMax"));
            return gson.toJson(fb);
        }
        if (updateAlarmVo.getHumidityMin()==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"humidityMin"));
            return gson.toJson(fb);
        }
        if(updateAlarmVo.getHumidityMin()>updateAlarmVo.getHumidityMax()){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"wrongHumidity"));
            return gson.toJson(fb);
        }
        if (updateAlarmVo.getTilt()==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"tilt"));
            return gson.toJson(fb);
        }
        if (updateAlarmVo.getShock()==null){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"shock"));
            return gson.toJson(fb);
        }
        if (alarmService.getByTypeAndScene(updateAlarmVo.getAlarmType(),updateAlarmVo.getAlarmType(),user.getCompanyName())!=null&&!updateAlarmVo.getAlarmType().equals(updateAlarmVo.getAlarmType())&&!updateAlarmVo.getAlarmScene().equals(updateAlarmVo.getAlarmScene())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"modifiedAlarmExist"));
            return gson.toJson(fb);
        }
        updateAlarmVo.setCompanyName(user.getCompanyName());
        int a=alarmService.updateAlarm(updateAlarmVo);
        if (a>0){
            fb.setCode(1);
            fb.setMessage(i18n.getMessage(request,"modifiedSuccess"));
        }else{
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"modifiedFailure"));
        }
        return gson.toJson(fb);

    }

    //删除预警信息
    @ApiOperation(value = "删除预警信息",notes = "删除预警信息,companyName不用传，传值说明：" +
            "   alarmType：需要删除的预警类型，" +
            "   alarmScene：需要删除的预警场景" )
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/delAlarm",method = RequestMethod.POST)
    public Object delAlarm(@RequestBody DelAlarmVo delAlarmVo,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        String  port = request.getHeader("AccountLanguage");
        if (StringUtils.isEmpty(delAlarmVo.getAlarmScene())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"deletedAlarmScene"));
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(delAlarmVo.getAlarmType())){
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"deletedAlarmType"));
            return gson.toJson(fb);
        }
        delAlarmVo.setCompanyName(user.getCompanyName());
        if("运输".equals(delAlarmVo.getAlarmScene())){
            if(tdlLogService.getByAlarm(delAlarmVo).size()>0){
                fb.setCode(0);
                fb.setMessage(i18n.getMessage(request,"dusedAlarm"));
                return gson.toJson(fb);
            }
        }else{
            if (warehouseService.getByAlarm(delAlarmVo).size()>0){
                fb.setCode(0);
                fb.setMessage(i18n.getMessage(request,"dusedAlarm"));
                return gson.toJson(fb);
            }
        }
        int a=alarmService.delAlarm(delAlarmVo);
        if (a>0){
            fb.setCode(1);
            fb.setMessage(i18n.getMessage(request,"deletedSuccess"));
        }else{
            fb.setCode(0);
            fb.setMessage(i18n.getMessage(request,"deletedFailure"));
        }
        return gson.toJson(fb);
    }

    //查询仓库场景下的预警类型
    @ApiOperation(value = "查询仓库场景下的预警类型",notes = "查询仓库场景下的预警类型")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/getForWarehouse",method = RequestMethod.GET)
    public Object getForWarehouse(HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        return alarmService.getForWarehouse(user.getCompanyName());

    }



    @ApiOperation(value = "查询运输场景下的预警类型",notes = "查询运输场景下的预警类型")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    @RequestMapping(value="/getForTransportation",method = RequestMethod.GET)
    public Object getForTransportation(HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        return alarmService.getForTransportation(user.getCompanyName());
    }

}
