package com.example.tdl.web;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.example.tdl.domain.dto.CommFeedback;
import com.example.tdl.domain.vo.*;
import com.example.tdl.service.*;
import com.example.tdl.service.redis.AlarmRedisService;
import com.example.tdl.service.redis.TokenRedisService;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/equipmentBinding")
public class EquipmentBindingController {
    private Logger logger = LoggerFactory.getLogger(this.getClass());

    CommFeedback fb = new CommFeedback();

    Gson gson = new Gson();

    @Autowired
    private ConfigService configService;

    @Autowired
    private GatewayService gatewayService;

    @Autowired
    private TDLDeviceService tdlDeviceService;

    @Autowired
    private AlarmRedisService alarmRedisService;

    @Autowired
    private AlarmService alarmService;

    @Autowired
    private TokenRedisService tokenRedisService;

    @Autowired
    private WarehouseService warehouseService;

    @Autowired
    private I18nController  i18n;

    List<String> list = new ArrayList<String>(){{
        add("T");
        add("h");
        add("a0");
        add("ta");
    }};

    @ApiOperation(value = "绑定gateWay和tdl",notes = "绑定gateWay和tdl" +
            "   SN：gateWay编号，" +
            "   type：gateWay类型，" +
            "   mode：1--运输/2--仓库，" +
            "   TDLSN：tdl编号，" +
            "   transportationNo：运输编号，" +
            "   warehouseNo：仓库编号," +
            "   alarmType:预警类型" )
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @PostMapping("/binding")
    public Object binding(@RequestBody BindingVo bindingVo,HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        if(StringUtils.isEmpty(bindingVo.getSN())){
            fb.setCode(0);
            fb.setMessage("网关编号不能为空");
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(bindingVo.getType())){
            fb.setCode(0);
            fb.setMessage("网关类型不能为空");
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(bindingVo.getTDLSN())){
            fb.setCode(0);
            fb.setMessage("Tdl编号不能为空");
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(bindingVo.getAlarmType())){
            fb.setCode(0);
            fb.setMessage("预警类型不能为空");
            return gson.toJson(fb);
        }
        if(bindingVo.getMode() == null){
            fb.setCode(0);
            fb.setMessage("使用场景不能为空");
            return gson.toJson(fb);
        }
        //获取gateWay的信息
        ResultGatewayVo resultGatewayVo =  gatewayService.getBySNAndType(bindingVo.getSN(),bindingVo.getType());
        if(resultGatewayVo == null){
            fb.setCode(0);
            fb.setMessage("该网关不存在");
            return gson.toJson(fb);
        }
        if(resultGatewayVo.getUseScene() !=0 && resultGatewayVo.getUseScene() !=bindingVo.getMode()){
            fb.setCode(0);
            fb.setMessage("该网关所属环境与当前不符");
            return gson.toJson(fb);
        }
        //判断当前tdl是否有网关在使用
        ResultTDLDeviceVo resultTDLDeviceVo = tdlDeviceService.getByTDLSN(bindingVo.getTDLSN());
        if(!StringUtils.isEmpty(resultTDLDeviceVo.getGatewaySN())){
            fb.setCode(0);
            fb.setMessage("该TDL已经被使用");
            return gson.toJson(fb);
        }
        if(StringUtils.isEmpty(bindingVo.getTransportationNo()) && !StringUtils.isEmpty(bindingVo.getWarehouseNo())){
            bindingVo.setTransportationNo("");
        }else if(StringUtils.isEmpty(bindingVo.getWarehouseNo()) && !StringUtils.isEmpty(bindingVo.getTransportationNo())){
            bindingVo.setWarehouseNo("");
        }

        //开始绑定
        //获取gateway下的所有tdl的编号
        List<String> devList = tdlDeviceService.getByGatewaySN(bindingVo.getSN(),bindingVo.getType());
        devList.add("TDL-"+bindingVo.getTDLSN());
        ConfigCMDVo configCMDVo = new ConfigCMDVo("config",5,15,devList,bindingVo.getMode(),System.currentTimeMillis()/1000l);
        Map<Object,Object> map=new HashMap<>();
        map.put("companyName",user.getCompanyName());
        map.put("transportationNo",bindingVo.getTransportationNo());
        map.put("warehouseNo",bindingVo.getWarehouseNo());
        map.put("useScene",bindingVo.getMode());
        map.put("gSN",bindingVo.getSN());
        map.put("gType",bindingVo.getType());
        map.put("alarmType",bindingVo.getAlarmType());
        map.put("message",gson.toJson(configCMDVo));
        map.put("flag",false);
        map.put("resp",null);
        map.put("state",1);
        map.put("untie",false);
        map.put("TDLSN",bindingVo.getTDLSN());
        map.put("createTime",System.currentTimeMillis());
        configService.addConfig(map);
        Map<Object,Object> msg=new HashMap<>();
        msg.put("msg",map.get("msg"));
        if (msg.get("msg").equals("配置网关成功")){
            fb.setCode(1);
            fb.setMessage("配置网关成功");
            //根据报警类型货报警信息
            ResultAlarmVo alarmVo =  alarmService.getByTypeAndScene(bindingVo.getAlarmType(),"运输",user.getCompanyName());
            for(String alias :list){
                List<AlarmRuleVo> alarmRuleVos = new ArrayList<>();
                AlarmRuleVo alarmRuleVo = new AlarmRuleVo();
                if(alias.equals("T")){
                    alarmRuleVo.setMinval(String.valueOf(alarmVo.getTemMin()));
                    alarmRuleVo.setMaxval(String.valueOf(alarmVo.getTemMax()));
                    alarmRuleVo.setMode("twoway");
                    alarmRuleVo.setPriority("1");
                }else if(alias.equals("h")){
                    alarmRuleVo.setMinval(String.valueOf(alarmVo.getHumidityMin()));
                    alarmRuleVo.setMaxval(String.valueOf(alarmVo.getHumidityMax()));
                    alarmRuleVo.setMode("twoway");
                    alarmRuleVo.setPriority("1");
                }else if(alias.equals("ta")){
                    alarmRuleVo.setMaxval(String.valueOf(alarmVo.getTilt()));
                    alarmRuleVo.setMode("greater");
                    alarmRuleVo.setPriority("1");
                }else if(alias.equals("a0")){
                    alarmRuleVo.setMaxval(String.valueOf(alarmVo.getShock()));
                    alarmRuleVo.setMode("greater");
                    alarmRuleVo.setPriority("1");
                }
                alarmRuleVos.add(alarmRuleVo);
                //向redis中写入报警信息
                alarmRedisService.hmSet("TDL/"+bindingVo.getType()+"/" +bindingVo.getSN()+"/Data","TDL-" +bindingVo.getTDLSN()+"_" +alias,gson.toJson(alarmRuleVos));
            }
        }else{
            fb.setCode(0);
            fb.setMessage(msg.get("msg").toString());
        }
        return gson.toJson(fb);
    }




    @ApiOperation(value = "重置tdl",notes = "重置tdl")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @PostMapping("/untie")
    public Object untie(@RequestBody String TDLSN){
        if(StringUtils.isEmpty(TDLSN)){
            fb.setCode(0);
            fb.setMessage("TDL序列号不能为空");
            return  gson.toJson(fb);
        }
        JSONObject jsonObject= JSON.parseObject(TDLSN);
        TDLSN=(String)jsonObject.get("TDLSN");
        //判断当前tdl是否有网关在使用
        ResultTDLDeviceVo resultTDLDeviceVo = tdlDeviceService.getByTDLSN(TDLSN);
        List<String> devList = tdlDeviceService.getByGatewaySN(resultTDLDeviceVo.getGatewaySN(),resultTDLDeviceVo.getGatewayType());
        ConfigVo configVo =configService.getConfig(resultTDLDeviceVo.getGatewaySN(),resultTDLDeviceVo.getGatewayType());
        ConfigCMDVo config = gson.fromJson(configVo.getMessage(),ConfigCMDVo.class);
        ConfigCMDVo configCMDVo ;
        if(devList.size()>1){
            devList.remove(TDLSN);
            configCMDVo = new ConfigCMDVo("config",config.getGpsPeriod(),config.getGprsPeriod(),devList,config.getMode(),System.currentTimeMillis()/1000l);
        }else{
            configCMDVo = new ConfigCMDVo();
        }
        Map<Object,Object> map=new HashMap<>();
        map.put("gSN",resultTDLDeviceVo.getGatewaySN());
        map.put("gType",resultTDLDeviceVo.getGatewayType());
        map.put("message",gson.toJson(configCMDVo));
        map.put("untie",false);
        map.put("TDLSN",TDLSN);
        map.put("endTime",System.currentTimeMillis());
        configService.updateConfig(map);
        Map<Object,Object> msg=new HashMap<>();
        msg.put("msg",map.get("msg"));
        if (msg.get("msg").equals("解除网关成功")){
            fb.setCode(1);
            fb.setMessage("解除tdl成功");
            String key ="TDL/"+resultTDLDeviceVo.getGatewayType()+"/" +resultTDLDeviceVo.getGatewaySN() +"/Data";
            alarmRedisService.delKey(key);
        }else{
            fb.setCode(0);
            fb.setMessage("解除tdl失败");
        }
        return  gson.toJson(fb);
    }


    @ApiOperation(value = "解绑网关",notes = "解绑网关")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
    })
    @PostMapping("/untieGateway")
    public  Object untieGateway(@RequestBody AddWarehouseGatewayVo addWarehouseGatewayVo,HttpServletRequest request){
        if(StringUtils.isEmpty(addWarehouseGatewayVo.getWarehouseNo())){
            fb.setCode(0);
            fb.setMessage("仓库编号不能为空");
            return  gson.toJson(fb);
        }
        if(warehouseService.getWarehouseNo(addWarehouseGatewayVo.getWarehouseNo()) ==0){
            fb.setCode(0);
            fb.setMessage("仓库编号不存在");
            return gson.toJson(fb);
        }
        if (StringUtils.isEmpty(addWarehouseGatewayVo.getSN())){
            fb.setCode(0);
            fb.setMessage("网关编号不能为空");
            return gson.toJson(fb);
        }
        if( gatewayService.getBySN(addWarehouseGatewayVo.getSN())==null){
            fb.setCode(0);
            fb.setMessage("该网关不存在");
            return gson.toJson(fb);
        }
        //判断该网关是否属于当前仓库
        Integer i =  warehouseService.getByWarehouseNoAndSN(addWarehouseGatewayVo.getWarehouseNo(),addWarehouseGatewayVo.getSN());
        if(i == 0){
            fb.setCode(0);
            fb.setMessage("该网关不属于该仓库");
            return gson.toJson(fb);
        }
        Map<Object,Object> map=new HashMap<>();
        map.put("warehouseNo",addWarehouseGatewayVo.getWarehouseNo());
        map.put("gSN",addWarehouseGatewayVo.getSN());
        map.put("gType","WTD93LG");
        warehouseService.untieGateway(map);
        Map<Object,Object> msg=new HashMap<>();
        msg.put("msg",map.get("msg"));
        if(msg.get("msg").equals("删除网关成功")){
            fb.setCode(1);
            fb.setMessage(msg.get("msg").toString());
        }else{
            fb.setCode(0);
            fb.setMessage(msg.get("msg").toString());
        }
        return  gson.toJson(fb);
    }
}
