package com.example.tdl.web;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.example.tdl.domain.dto.CommFeedback;
import com.example.tdl.domain.vo.*;
import com.example.tdl.service.AlarmLogService;
import com.example.tdl.service.CircuitService;

import com.example.tdl.service.TDLLogService;
import com.example.tdl.service.redis.InfoRedisService;
import com.example.tdl.service.redis.TokenRedisService;
import com.google.gson.Gson;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.influxdb.dto.Query;
import org.influxdb.dto.QueryResult;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.influxdb.InfluxDBTemplate;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@RestController
@RequestMapping("/homePage")
public class HomePageController {
    private Logger logger = LoggerFactory.getLogger(HomePageController.class);

    Gson gson = new Gson();

    CommFeedback fb = new CommFeedback();

    @Autowired
    private CircuitService circuitService;

    @Autowired
    private AlarmLogService alarmLogServcie;

    @Autowired
    private TDLLogService tdlLogService;

    @Autowired
    private InfluxDBTemplate influxDBTemplate;

    @Autowired
    private TokenRedisService tokenRedisService;

    @Autowired
    private InfoRedisService infoRedisService;

    private final static String database ="original";


    @Autowired
    private I18nController  i18n;

    //获取当天所有的设备数据
    @ApiOperation(value = "获取所有的运输数据",notes = "获取运输设备数据：" +
            "sum:总数量;" +
            "onRoute:在途数量;" +
            "delay:延误数量;" +
            "alarmed: 报警数量;" +
            "bump：颠簸数量;" +
            "list: " +
            "{classify:报警描述;"+
            "count:数量;}"+
            "alarmLogVoList :"+
            "{description:报警描述;" +
            "data:数值;" +
            "alarmTime:报警时间;}" +
            "circuitList：" +
            "{transportationNo:运输编号;" +
            "cargoNo：货物编号;" +
            "startCity:开始城市;" +
            "endCity:目的城市;" +
            "startTime:开始时间;" +
            "endTime:到达时间;" +
            "lng:当前经度;" +
            "lat:当前纬度;" +
            "time:当前时间;}" +
            "histogramDataList{" +
            "   classify: 种类（1-温度，2-湿度，3-震动，4-倾角，5-延误，6-离线）" +
            "   count：数量}")
    @PostMapping("/getAll")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    public Object getAll(@RequestBody String time, HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        JSONObject jsonObject= JSON.parseObject(time);
        time=(String) jsonObject.get("time");
        time = time.replace("天","").trim();
        //获取数量
        HomePageDataVo dataVo = new HomePageDataVo();
        //获取所有的线路数量
        dataVo.setSum(circuitService.getCount(time,System.currentTimeMillis()/1000l,user.getCompanyName()));//总数量
        //获取近几天的线路信息
        dataVo.setOnRoute(circuitService.getOnRoute(user.getCompanyName()));//在途数量
       Integer delay =  circuitService.getDelayCount(System.currentTimeMillis(),time,user.getCompanyName());
        dataVo.setDelay(delay);//延误数量
        dataVo.setAlarmed( alarmLogServcie.getCount(time,user.getCompanyName()));//报警数量
        dataVo.setBump(alarmLogServcie.getBumpCount(time,user.getCompanyName())); //获取颠簸数量
        dataVo.setAlarmLogVoList(alarmLogServcie.getAlarmLog(time,user.getCompanyName()));//报警列表
        List<HistogramData> list = alarmLogServcie.getByCount(time,user.getCompanyName());//获取报警柱状图
        for(int j = 0,length = list.size();j<length;j++){
            if(list.get(j).getClassify().equals("5")){
                Integer count = list.get(j).getCount() +delay;
                list.get(j).setCount(count);
            }
        }
        //获取设备的当前位置
        List<ResultCircuitVo> circuitVos =  circuitService.getByTime(user.getCompanyName());
        List<CircuitVo> circuitList = new ArrayList<>();
        Integer num = 0;
        for(int i = 0;i<circuitVos.size();i++){
            CircuitVo circuitVo = new CircuitVo();
            circuitVo.setTransportationNo(circuitVos.get(i).getTransportationNo());
            circuitVo.setCargoNo(circuitVos.get(i).getCargoNo());
            circuitVo.setStartCity(circuitVos.get(i).getStartCity());
            circuitVo.setEndCity(circuitVos.get(i).getEndCity());
            circuitVo.setStartTime(circuitVos.get(i).getStartTime());
            circuitVo.setEndTime(circuitVos.get(i).getEndTime());
            ResultGatewayVo gatewayVo  = tdlLogService.getByNo(circuitVos.get(i).getTransportationNo());
            String device = gatewayVo.getType() + "_" + gatewayVo.getSN();
            Object data = infoRedisService.getHash("DeviceOnline",   device );
            if(data !=null){
                OnlineVo onlineVo =  gson.fromJson(data.toString(),OnlineVo.class);
                if(onlineVo.getOnline() == 0){
                    num =num +1;
                }
            }
            if (circuitVos.get(i).getStartTime() == null || circuitVos.get(i).getStartTime() == 0) {
                circuitList.add(circuitVo);
            } else {
                if(gatewayVo==null){
                    circuitList.add(circuitVo);
                }else {
                    Long startTime = circuitVos.get(i).getStartTime() * 1000000l;
                    String sql = "";
                    if (circuitVos.get(i).getEndTime() == null || circuitVos.get(i).getEndTime() == 0) {
                        sql = "SELECT \"lng\",\"lat\" FROM \"tdl_policy\".\"" + device + "\"where  \"lng\"!=0 and \"lat\"!=0 and  time >= " + startTime + " ORDER BY time  desc limit 1";
                    } else {
                        sql = "SELECT \"lng\",\"lat\" FROM \"tdl_policy\".\"" + device + "\"where \"lng\"!=0 and \"lat\"!=0 and  time >= " + startTime + " and time<= " + circuitVos.get(i).getEndTime() * 1000000l + " ORDER BY time  desc limit 1";
                    }
                    QueryResult queryResult = influxDBTemplate.query(new Query(sql, database));
                    if (queryResult.getResults().get(0).getSeries() != null) {
                        circuitVo.setTime(parseTime(queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(0).toString()));
                        circuitVo.setLng(queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(1).toString());
                        circuitVo.setLat(queryResult.getResults().get(0).getSeries().get(0).getValues().get(0).get(2).toString());
                    }
                    circuitList.add(circuitVo);
                }
            }
        }
        list.add(new HistogramData("6",num));
        dataVo.setHistogramDataList(list);
        dataVo.setCircuitList(circuitList);
        return gson.toJson(dataVo);
    }



    //获取当天所有的设备数据
    @ApiOperation(value = "获取所有的运输数据",notes = "获取运输设备数据：" +
            "onRoute:在途数量;" +
            "delay:延误数量;" +
            "alarmed: 报警数量;")
    @PostMapping("/getAllForAPP")
    @ApiImplicitParams({
            @ApiImplicitParam(paramType="header", name = "Account_token", value = "token", required = true, dataType = "String"),
            @ApiImplicitParam(paramType="header", name = "AccountLanguage", value = "language", required = false, dataType = "String"),
    })
    public Object getAllForAPP(HttpServletRequest request){
        String token = request.getHeader("Account_token");
        String datum = tokenRedisService.get("TOKEN_" +token);
        UserRedisVo user =  gson.fromJson(datum,UserRedisVo.class);
        AlarmDataVo alarmDataVo = new AlarmDataVo();
        alarmDataVo.setOnRoute(circuitService.getOnRoute(user.getCompanyName()));
        alarmDataVo.setAlarmed(alarmLogServcie.getAlarmCount(user.getCompanyName()));
        alarmDataVo.setDelay(circuitService.getDelay(System.currentTimeMillis(),user.getCompanyName()));
        return gson.toJson(alarmDataVo);
    }


    //解析时间
    public String parseTime(String time){
        String dateResult =null;
        try {
            dateResult = dateToStamp(time);
        } catch (ParseException e) {
            e.printStackTrace();
            logger.info(e.toString());
            fb.setCode(0);
            fb.setMessage("解析时间异常");
        }
        return dateResult;
    }

    //将utc时间转换成时间戳
    public static String dateToStamp(String s) throws ParseException {
        Date date =null ;
        if(s.length() !=20 ) {
            date = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSS'Z'").parse(s);
        }else{
            date = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'").parse(s);
        }
        return String.valueOf(date.getTime());
    }
}
